#!/usr/bin/env tclsh

package require sqlite3

set Datenbank [file join [file dirname [info script]] MeineDB] ; # entfaellt, wenn die Datenbank im Arbeitsspeicher gehalten wird

proc DatenbankErstellen {} {
  global db

  db eval {create table Person( \
    ID integer primary key autoincrement, \
    Name text, \
    AlterJahre integer, \
    IDAdresse integer
  )}
  
  db eval {create table Adresse( \
    ID integer primary key autoincrement, \
    Strasse text, \
    PLZ text, \
    Ort text
  )}
}

proc DatenbankOeffnen {} {
  global Datenbank
  global db
  if {[file exist $Datenbank] == 0} {
    # Datenbank existiert nicht
    sqlite3 db $Datenbank ; # Alternativ: sqlite3 db ":memory:"
    DatenbankErstellen
  } else {
    # Datenbank existiert
    sqlite3 db $Datenbank
  }
}

proc DatenbankSchliessen {} {
  global db
  catch db close
}

proc DatenbankLoeschen {} {
  global db
  db eval {drop table Person}
  db eval {drop table Adresse}
}

proc PersonEinfuegen {Name Alter Adresse} {
  global db
  db eval {insert into Person values(NULL,$Name,$Alter,$Adresse)}
}

proc AdresseEinfuegen {Strasse PLZ Ort} {
  global db
  db eval {insert into Adresse values(NULL,$Strasse,$PLZ,$Ort)}
}

proc PersonAnzeigen {ID} {
  global db
  puts [db eval {select Person.Name,Person.AlterJahre,Adresse.Strasse,Adresse.PLZ,Adresse.Ort from Person left outer join Adresse ON Person.IDAdresse=Adresse.ID where Person.ID=$ID}]
}

proc PersonenAnzeigenUndZaehlen {} {
  global db
  puts "\n"
  puts "Prozedur PersonenAnzeigenUndZaehlen:"
  set AnzahlPersonenBis39 0
  set AnzahlPersonenAb40 0
  db eval {select Person.Name,Person.AlterJahre,Adresse.Strasse,Adresse.PLZ,Adresse.Ort from Person left outer join Adresse ON Person.IDAdresse=Adresse.ID} Ergebnis {
    parray Ergebnis
    if {$Ergebnis(AlterJahre)<40} {
      incr AnzahlPersonenBis39
    } else {
      incr AnzahlPersonenAb40
    }
  }
  puts "Personen bis 39 Jahre: $AnzahlPersonenBis39"
  puts "Personen ab 40 Jahre: $AnzahlPersonenAb40"
  puts "\n"
}

proc EinfuegenOhneTransaction {} {
  global db
  for {set i 0} {$i <=300} {incr i} {
    AdresseEinfuegen "Neue Str. 9" 65432 Musterstadt
  }
}

proc EinfuegenMitTransaction {} {
  global db
  db eval {begin transaction}
  for {set i 0} {$i <=300} {incr i} {
    AdresseEinfuegen "Neue Str. 9" 65432 Musterstadt
  }
  db eval {end transaction}
}

# Hauptprogramm

DatenbankOeffnen

db eval {begin transaction}
AdresseEinfuegen "Hauptstr. 5" 65432 Musterstadt
AdresseEinfuegen "Nebenstr. 7" 23456 Neustadt
db eval {end transaction}
 
db eval {begin transaction}
PersonEinfuegen Anton 38 1
PersonEinfuegen Berta 36 1
PersonEinfuegen Caesar 41 2
db eval {end transaction}

PersonAnzeigen 1
PersonAnzeigen 2
PersonAnzeigen 3

PersonenAnzeigenUndZaehlen

puts "Einfuegen ohne Transaktion:"
set Start [clock clicks -milliseconds]
EinfuegenOhneTransaction
set Stopp [clock clicks -milliseconds]
set Dauer [expr ($Stopp - $Start)/1000.0]
puts "Dauer: $Dauer Sekunden"

puts "Einfuegen mit Transaktion:"
set Start [clock clicks -milliseconds]
EinfuegenMitTransaction
set Stopp [clock clicks -milliseconds]
set Dauer [expr ($Stopp - $Start)/1000.0]
puts "Dauer: $Dauer Sekunden"

DatenbankSchliessen

file delete -force $Datenbank
