#!/usr/bin/env wish

proc GroesseFormatieren {Groesse} {
	if {$Groesse > 1048576} {
		set GroesseFormatiert "[expr round(1.0*$Groesse/1048576)] M"
	} elseif {$Groesse > 1024} {
		set GroesseFormatiert "[expr round(1.0*$Groesse/1024)] K"
	} else {
		set GroesseFormatiert "$Groesse B"
	}
	return $GroesseFormatiert
}

proc KnotenOeffnen {ID} {
	if {[.tv exists Dummy$ID]} {
		.tv delete Dummy$ID
	}
	set Startordner [.tv item $ID -text]
	set Dateien [lsort -nocase [glob -nocomplain -directory $Startordner -types {f} *]]
	foreach Element $Dateien {
		set Name [file tail $Element]
		set Groesse [GroesseFormatieren [file size $Element]]
		set Datum [clock format [file mtime $Element] -format {%Y-%m-%d %H:%M:%S}]
		set Eintrag {}
		lappend Eintrag $Name
		lappend Eintrag $Groesse
		lappend Eintrag $Datum
		set tmpID [.tv insert $ID end -values $Eintrag]
	}
	set Ordner [lsort -nocase [glob -nocomplain -directory $Startordner -types {d} *]]
	foreach Element $Ordner {
		set tmpID [.tv insert $ID end -text $Element]
		.tv insert $tmpID end -id Dummy$tmpID -text ""
	}
}

proc KnotenSchliessen {ID} {
	set Elemente [.tv children $ID]
	foreach Element $Elemente {
		if {[.tv exists $Element]} {
			.tv delete $Element
		}
	}
	.tv insert $ID end -id Dummy$ID -text ""
}

set Startordner $env(HOME)

ttk::treeview .tv -columns {Datei Groesse Datum} -yscrollcommand {.sbY set} -xscrollcommand {.sbX set}
.tv column #0 -minwidth 200 -stretch 1
.tv column Datei -minwidth 200 -stretch 1
.tv column Groesse -minwidth 100 -stretch 1
.tv column Datum -minwidth 150 -stretch 1

.tv heading #0 -text "Ordner"
.tv heading Datei -text "Datei"
.tv heading Groesse -text "Groesse"
.tv heading Datum -text "Datum Uhrzeit"

.tv heading #0 -anchor w
.tv heading Datei -anchor w
.tv heading Groesse -anchor c
.tv heading Datum -anchor c

.tv column #0 -anchor w
.tv column Datei -anchor w
.tv column Groesse -anchor c
.tv column Datum -anchor c

set ID [.tv insert {} end -text $Startordner]
.tv insert $ID end -id Dummy$ID -text ""

ttk::scrollbar .sbY -command {.tv yview}
ttk::scrollbar .sbX -orient horizontal -command {.tv xview}

pack .sbY -side right -fill y
pack .sbX -side bottom -fill x
pack .tv -side left -expand yes -fill both
bind .tv <<TreeviewOpen>> {KnotenOeffnen [.tv selection]}
bind .tv <<TreeviewClose>> {KnotenSchliessen [.tv selection]}
