#!/usr/bin/env tclsh

package require fileutil
package require Tktable

proc Sortieren {tmpVar Zeilen Spalten Sortierspalte Sortierung Art} {
  # Sortiert die Tabelle
  # Da die Tabellenvariable ein Array ist und 
  # man ein Array nicht sortieren kann, wird 
  # das Array in eine Liste umgewandelt,
  # dann die Liste sortiert und schliesslich 
  # die Liste in ein Array zurueck umgewandelt.
  #
  # tmpVar = Name der Tabellen-Variable
  # Zeilen = Anzahl der Zeilen in der Tabelle
  # Spalten = Anzahl der Spalten in der Tabelle
  # Sortierspalte = Index der Spalte, nach der sortiert wird
  # Sortierung = auf- oder absteigend sortieren
  # Art = numerisch oder alphanumerisch sortieren

  # Verweis auf die Tabellen-Variable
  upvar $tmpVar Tabelle

  # Programm fuer weitere Aktionen sperren
  tk busy hold . 
  tk busy configure . -cursor watch
  update
  
  # Tabellen-Variable (Array) in Liste umwandeln (ohne Titelzeile)
  set Liste {}
  for {set i 1} {$i < $Zeilen} {incr i} {
    # Alle Werte in der Zeile zu einer Liste zusammenfuegen
    set ListeZeilenwerte {}
    for {set j 0} {$j < $Spalten} {incr j} {
      lappend ListeZeilenwerte $Tabelle($i,$j)
    }
    # Die Liste mit den Zeilenwerten der (Gesamt-)Liste hinzufuegen.
    # Es handelt sich also um Listen innerhalb einer Liste.
    lappend Liste $ListeZeilenwerte
  }

  # Liste sortieren
  set Liste [lsort $Art $Sortierung -index $Sortierspalte $Liste]

  # Liste in Tabellen-Variable umwandeln
  for {set i 1} {$i < $Zeilen} {incr i} {
    # weil die Liste mit Index=0 startet, die Werte aber in Zeile 1 der
    # Tabelle angezeigt werden sollen, muss der Index der Liste
    # gegenueber der Tabelle um 1 reduziert werden.
    set k [expr $i - 1]
    for {set j 0} {$j < $Spalten} {incr j} {
      # Die innerste Klammer holt die gesamte Zeile mit den Werten,
      # die aeussere Klammer holt aus dieser Zeile die Werte zu
      # jeder Spalte. 
      set Tabelle($i,$j) [lindex [lindex $Liste $k] $j]
    }
  }

  # Sortierung wechseln
  if {$Sortierung eq "-increasing"} {
    set Sortierung "-decreasing"
  } else {
    set Sortierung "-increasing"
  }

  # Sperrung des Programms aufheben
  tk busy forget .
  update

  return $Sortierung
}

proc DateienEinlesenVergleich {Datei} {
  # Definiert, welche Dateien eingelesen werden sollen.
  # In diesem Beispiel sollen alle Dateien eingelesen werden.
  return [string match * $Datei]
}

proc DateienEinlesen {tmpVar Ordner} {
  # Liest die Dateien ein

  # Verweis auf die Tabellen-Variable
  upvar $tmpVar Tabelle
	
  # Prueft, ob der Ordner existiert
  if {![file exists $Ordner]} {return}

  # Programm fuer weitere Aktionen sperren
  tk busy hold . 
  tk busy configure . -cursor watch
  update
  
  # Tabelleninhalt loeschen
  array unset Tabelle

  # Tabellenueberschrift
  set Tabelle(0,0) "Datei"
  set Tabelle(0,1) "Groesse"
  set Tabelle(0,2) "Datum"
	
  # Dateien einlesen
  set Dateien [fileutil::find $Ordner DateienEinlesenVergleich]
	
  # Dateien mit den Dateieigenschaften in die Tabelle uebernehmen
  set Zeile 1
  foreach Datei $Dateien {
    if {[file isfile $Datei]} {
      set Groesse [file size $Datei]
      if {$Groesse >= 1048576} {
        set FormatierteGroesse "[expr round(double($Groesse)/1048576.0*10.0)/10.0] M"
      } elseif {$Groesse >= 1024} {
        set FormatierteGroesse "[expr round(double($Groesse)/1024.0*10.0)/10.0] K"
      } else {
        set FormatierteGroesse "$Groesse B"
      }
      
      set Zeit [file mtime $Datei]
      set FormatierteZeit [clock format $Zeit -format {%Y-%m-%d %H:%M:%S}]
			
      set Tabelle($Zeile,0) $Datei
      set Tabelle($Zeile,1) $FormatierteGroesse
      set Tabelle($Zeile,2) $FormatierteZeit
      set Tabelle($Zeile,3) $Groesse
      set Tabelle($Zeile,4) $Zeit
      incr Zeile
    }
  }
	
  # Zeilenanzahl der Tabelle anpassen
  .frTabelle.tbTabelle configure -rows $Zeile
  
  # Sperrung des Programms aufheben
  tk busy forget .
  update
}

proc OrdnerWaehlen {Ordner} {
  # Auswahl des Startordners
  set tmp [tk_chooseDirectory]
  if {$tmp ne ""} {
    set Ordner $tmp
  }
  return $Ordner
}

# Ordnerauswahl erzeugen und anzeigen
set Startordner $env(HOME)
ttk::frame .frOrdner
ttk::entry .frOrdner.enOrdner -width 10 -textvariable Startordner
ttk::button .frOrdner.btOrdner -text "..." -command {
  set Startordner [OrdnerWaehlen $Startordner]
  .frOrdner.btStart invoke
}
ttk::button .frOrdner.btStart -text "Start" -command {DateienEinlesen Tabelle $Startordner}
pack .frOrdner.btStart -side left -padx 2
pack .frOrdner.enOrdner -side left -padx 2 -fill x -expand yes
pack .frOrdner.btOrdner -side left -padx 2
pack .frOrdner -side top -fill x

# Return-Taste mit Start-Button verbinden.
bind .frOrdner.enOrdner <Return> {.frOrdner.btStart invoke}

# Tabelle definieren
ttk::frame .frTabelle
table .frTabelle.tbTabelle \
  -titlerows 1 \
  -titlecols 0 \
  -cols 3 \
  -rows 1 \
  -height 20 \
  -width 3 \
  -rowheight 1 \
  -colwidth 15 \
  -maxheight 300 \
  -maxwidth 600 \
  -xscrollcommand {.frTabelle.xscroll set} \
  -yscrollcommand {.frTabelle.yscroll set} \
  -variable Tabelle

# Interne Spaltenbezeichnung definieren
.frTabelle.tbTabelle tag col SpalteName 0
.frTabelle.tbTabelle tag col SpalteGroesse 1
.frTabelle.tbTabelle tag col SpalteZeit 2

# Spaltenbreite definieren
.frTabelle.tbTabelle width 0 50
.frTabelle.tbTabelle width 1 10
.frTabelle.tbTabelle width 2 50

# Spaltenausrichtung definieren
.frTabelle.tbTabelle tag configure SpalteName -anchor w
.frTabelle.tbTabelle tag configure SpalteGroesse -anchor center
.frTabelle.tbTabelle tag configure SpalteZeit -anchor center

# Scroll-Leisten
ttk::scrollbar .frTabelle.xscroll -command {.frTabelle.tbTabelle xview} -orient horizontal
ttk::scrollbar .frTabelle.yscroll -command {.frTabelle.tbTabelle yview} -orient vertical

# Tabelle anzeigen
pack .frTabelle -side top -fill both -expand yes
pack .frTabelle.xscroll -side bottom -fill x
pack .frTabelle.yscroll -side right -fill y
pack .frTabelle.tbTabelle -side top -fill both -expand yes

# Buttons erzeugen und anzeigen
ttk::frame .frButton
ttk::button .frButton.btName -text "Name" -command {set NameSortierung [Sortieren Tabelle [.frTabelle.tbTabelle cget -rows] 5 0 $NameSortierung "-dictionary"]}
ttk::button .frButton.btGroesse -text "Groesse" -command {set GroesseSortierung [Sortieren Tabelle [.frTabelle.tbTabelle cget -rows] 5 3 $GroesseSortierung "-integer"]}
ttk::button .frButton.btZeit -text "Zeit" -command {set ZeitSortierung [Sortieren Tabelle [.frTabelle.tbTabelle cget -rows] 5 4 $ZeitSortierung "-integer"]}
pack .frButton.btName -side left
pack .frButton.btGroesse -side left
pack .frButton.btZeit -side left
pack .frButton -side bottom

# Sortierung (auf-/absteigend) initialisieren
set NameSortierung "-increasing"
set GroesseSortierung "-decreasing"
set ZeitSortierung "-decreasing"

# Start-Button aufrufen
.frOrdner.btStart invoke
