#!/usr/bin/env tclsh

package require fileutil
package require Tktable

proc ListeSortieren {Liste Sortierspalte Sortierung Art} {
  # Sortiert die Liste
  # Sortierspalte = Index der Spalte, nach der sortiert wird
  # Sortierung = auf- oder absteigend sortieren
  # Art = numerisch oder alphanumerisch sortieren

  # Programm fuer weitere Aktionen sperren
  tk busy hold . 
  tk busy configure . -cursor watch
  update
  
  # Liste sortieren
  set Liste [lsort $Art $Sortierung -index $Sortierspalte $Liste]

  # Sperrung des Programms aufheben
  tk busy forget .
  update

  return $Liste
}

proc DateienEinlesenVergleich {Datei} {
  # Definiert, welche Dateien eingelesen werden sollen.
  # In diesem Beispiel sollen alle Dateien eingelesen werden.
  return [string match * $Datei]
}

proc DateienEinlesen {Ordner} {
  # Liest die Dateien ein

  # Prueft, ob der Ordner existiert
  if {![file exists $Ordner]} {return}

  # Programm fuer weitere Aktionen sperren
  tk busy hold . 
  tk busy configure . -cursor watch
  update
  
  # Leere Liste erzeugen
  set Liste {}
	
  # Dateien einlesen
  set Dateien [fileutil::find $Ordner DateienEinlesenVergleich]
	
  # Dateien mit den Dateieigenschaften in die Liste uebernehmen
  foreach Datei $Dateien {
    if {[file isfile $Datei]} {
      set Zeile {}
      set Groesse [file size $Datei]
      if {$Groesse >= 1048576} {
        set FormatierteGroesse "[expr round(double($Groesse)/1048576.0*10.0)/10.0] M"
      } elseif {$Groesse >= 1024} {
        set FormatierteGroesse "[expr round(double($Groesse)/1024.0*10.0)/10.0] K"
      } else {
        set FormatierteGroesse "$Groesse B"
      }
      set Zeit [file mtime $Datei]
      set FormatierteZeit [clock format $Zeit -format {%Y-%m-%d %H:%M:%S}]
			
      # Die Dateieigenschaften als Liste speichern (wird spaeter als Zeile in der Tabelle angezeigt)
      set Zeile {}
      lappend Zeile $Datei $FormatierteGroesse $FormatierteZeit $Groesse $Zeit
      
      # Die Zeile der Liste hinzufuegen (es ensteht somit eine Liste, die Listen enthaelt)
      lappend Liste $Zeile
	}
  }
  
  # Sperrung des Programms aufheben
  tk busy forget .
  update

  return $Liste
}

proc OrdnerWaehlen {Ordner} {
  # Auswahl des Startordners
  set tmp [tk_chooseDirectory]
  if {$tmp ne ""} {
    set Ordner $tmp
  }
  return $Ordner
}

proc ListeInArrayUmwandeln {tmpVar Liste} {
  # Verweis auf die Tabellen-Variable
  upvar $tmpVar Tabelle

  # Tabelleninhalt loeschen
  array unset Tabelle  	
  
  # Die Liste mit den Dateien und Dateieigenschaften in die Array-Variable 'Tabelle' uebertragen
  for {set i 1} {$i <= [llength $Liste]} {incr i} {
	set j [expr $i - 1]
	set Zeile [lindex $Liste $j]
    set Tabelle($i,0) [lindex $Zeile 0]
    set Tabelle($i,1) [lindex $Zeile 1]
    set Tabelle($i,2) [lindex $Zeile 2]
    set Tabelle($i,3) [lindex $Zeile 3]
    set Tabelle($i,4) [lindex $Zeile 4]
  }
  
  # Zeilenanzahl der Tabelle anpassen
  .frTabelle.tbTabelle configure -rows $i       
}

proc SortierungAendern {Sortierung} {
  if {$Sortierung eq "-increasing"} {
    set Sortierung "-decreasing"
  } else {
    set Sortierung "-increasing"
  }
  return $Sortierung
}

# Leere Tabelle erzeugen
array set Tabelle {}
  
# Leere Liste erzeugen
set Liste {}

# Tabellenueberschrift
set Tabelle(0,0) "Datei"
set Tabelle(0,1) "Groesse"
set Tabelle(0,2) "Datum"

# Startordner festlegen
set Startordner $env(HOME)

# GUI erstellen
ttk::frame .frOrdner
ttk::entry .frOrdner.enOrdner -width 10 -textvariable Startordner
ttk::button .frOrdner.btOrdner -text "..." -command {
  set Startordner [OrdnerWaehlen $Startordner]
  .frOrdner.btStart invoke
}
ttk::button .frOrdner.btStart -text "Start" -command {
  set Liste [DateienEinlesen $Startordner]
  ListeInArrayUmwandeln Tabelle $Liste
}
pack .frOrdner.btStart -side left -padx 2
pack .frOrdner.enOrdner -side left -padx 2 -fill x -expand yes
pack .frOrdner.btOrdner -side left -padx 2
pack .frOrdner -side top -fill x

# Return-Taste mit Start-Button verbinden.
bind .frOrdner.enOrdner <Return> {.frOrdner.btStart invoke}

# Tabelle definieren
ttk::frame .frTabelle
table .frTabelle.tbTabelle \
  -titlerows 1 \
  -titlecols 0 \
  -cols 3 \
  -rows 1 \
  -height 20 \
  -width 3 \
  -rowheight 1 \
  -colwidth 15 \
  -maxheight 300 \
  -maxwidth 600 \
  -xscrollcommand {.frTabelle.xscroll set} \
  -yscrollcommand {.frTabelle.yscroll set} \
  -variable Tabelle

# Interne Spaltenbezeichnung definieren
.frTabelle.tbTabelle tag col SpalteName 0
.frTabelle.tbTabelle tag col SpalteGroesse 1
.frTabelle.tbTabelle tag col SpalteZeit 2

# Spaltenbreite definieren
.frTabelle.tbTabelle width 0 50
.frTabelle.tbTabelle width 1 10
.frTabelle.tbTabelle width 2 50

# Spaltenausrichtung definieren
.frTabelle.tbTabelle tag configure SpalteName -anchor w
.frTabelle.tbTabelle tag configure SpalteGroesse -anchor center
.frTabelle.tbTabelle tag configure SpalteZeit -anchor center

# Scroll-Leisten
ttk::scrollbar .frTabelle.xscroll -command {.frTabelle.tbTabelle xview} -orient horizontal
ttk::scrollbar .frTabelle.yscroll -command {.frTabelle.tbTabelle yview} -orient vertical

# Tabelle anzeigen
pack .frTabelle -side top -fill both -expand yes
pack .frTabelle.xscroll -side bottom -fill x
pack .frTabelle.yscroll -side right -fill y
pack .frTabelle.tbTabelle -side top -fill both -expand yes

# Buttons erzeugen und anzeigen
ttk::frame .frButton
ttk::button .frButton.btName -text "Name" -command {
  set Liste [ListeSortieren $Liste 0 $NameSortierung "-dictionary"]
  ListeInArrayUmwandeln Tabelle $Liste
  set NameSortierung [SortierungAendern $NameSortierung]
}
ttk::button .frButton.btGroesse -text "Groesse" -command {
  set Liste [ListeSortieren $Liste 3 $GroesseSortierung "-integer"]
  ListeInArrayUmwandeln Tabelle $Liste
  set GroesseSortierung [SortierungAendern $GroesseSortierung]
}
ttk::button .frButton.btZeit -text "Zeit" -command {
  set Liste [ListeSortieren $Liste 4 $ZeitSortierung "-integer"]
  ListeInArrayUmwandeln Tabelle $Liste
  set ZeitSortierung [SortierungAendern $ZeitSortierung]
}
pack .frButton.btName -side left
pack .frButton.btGroesse -side left
pack .frButton.btZeit -side left
pack .frButton -side bottom

# Sortierung (auf-/absteigend) initialisieren
set NameSortierung "-increasing"
set GroesseSortierung "-decreasing"
set ZeitSortierung "-decreasing"

# Start-Button aufrufen
.frOrdner.btStart invoke
