#!/usr/bin/env wish

package require struct::tree

::struct::tree Baum

# Es wird folgender Baum erfasst:
# root -+- Ordner A
#       +- Ordner B -+- Ordner D -+- Datei 3
#       |            |            +- Datei 4
#       |            +- Datei 1
#       |            +- Datei 2
#       +- Ordner C

# Unterhalb von root:
Baum insert root end 0
Baum set 0 Name "Ordner A"
Baum set 0 Typ "Ordner"
Baum set 0 Groesse ""
Baum set 0 Datum ""

Baum insert root end 1
Baum set 1 Name "Ordner B"
Baum set 1 Typ "Ordner"
Baum set 1 Groesse ""
Baum set 1 Datum ""

Baum insert root end 2
Baum set 2 Name "Ordner C"
Baum set 2 Typ "Ordner"
Baum set 2 Groesse ""
Baum set 2 Datum ""

# Unterhalb von Ordner B:
Baum insert 1 end 3
Baum set 3 Name "Ordner D"
Baum set 3 Typ "Ordner"
Baum set 3 Groesse ""
Baum set 3 Datum ""

Baum insert 1 end 4
Baum set 4 Name "Datei 1"
Baum set 4 Typ "Datei"
Baum set 4 Groesse "114 kb"
Baum set 4 Datum "2019-03-24"

Baum insert 1 end 5
Baum set 5 Name "Datei 2"
Baum set 5 Typ "Datei"
Baum set 5 Groesse "987 kb"
Baum set 5 Datum "2018-10-05"

# Unterhalb von Ordner D:
Baum insert 3 end 6
Baum set 6 Name "Datei 3"
Baum set 6 Typ "Datei"
Baum set 6 Groesse "374 kb"
Baum set 6 Datum "2020-01-12"

Baum insert 3 end 7
Baum set 7 Name "Datei 4"
Baum set 7 Typ "Datei"
Baum set 7 Groesse "822 kb"
Baum set 7 Datum "2020-04-17"

#################

ttk::treeview .tv -columns {Typ Groesse Datum} -yscrollcommand {.sbY set} -xscrollcommand {.sbX set}

.tv column #0 -minwidth 200 -stretch 1
.tv column Typ -minwidth 100 -stretch 1
.tv column Groesse -minwidth 100 -stretch 1
.tv column Datum -minwidth 100 -stretch 1

.tv heading #0 -text "Name"
.tv heading Typ -text "Typ"
.tv heading Groesse -text "Groesse"
.tv heading Datum -text "Datum"

.tv insert {} end -id root -text "Root" ; # die ID muss "root" lauten, weil in struct::tree der erste Knoten die ID "root" hat.

ttk::scrollbar .sbY -command {.tv yview}
ttk::scrollbar .sbX -orient horizontal -command {.tv xview}

pack .sbY -side right -fill y
pack .sbX -side bottom -fill x
pack .tv -side left -expand yes -fill both

#################

Baum walk root -order pre -type bfs Id {
  set Name ""
  set Typ ""
  set Groesse ""
  set Datum ""
  
  if {[Baum keyexists $Id Name]} {
    set Name [Baum get $Id Name]
  }
  if {[Baum keyexists $Id Typ]} {
    set Typ [Baum get $Id Typ]
  }
  if {[Baum keyexists $Id Groesse]} {
    set Groesse [Baum get $Id Groesse]
  }
  if {[Baum keyexists $Id Datum]} {
    set Datum [Baum get $Id Datum]
  }

  set ElternId [Baum parent $Id]
  if {$Typ eq "Ordner" || $Typ eq "Datei"} {
    set Liste [list $Typ $Groesse $Datum]
    .tv insert $ElternId end -id $Id -text $Name -values $Liste
  }
}

