#!/usr/bin/env tclsh

package require struct::graph

proc KnotenAnzeigen {Modus Graph Knoten} {
  # Zeigt die Daten des Knoten an
  global Zeitformat
  global AngezeigteKnoten

  if {$Knoten in $AngezeigteKnoten} {return}

  puts "Knoten=$Knoten"
  puts "  Beschreibung:[$Graph node get $Knoten Beschreibung]"
  puts "  Person:[$Graph node get $Knoten Person]"
  set Beginn [$Graph node get $Knoten Beginn]
  puts "  Beginn:[clock format $Beginn -format $Zeitformat]"
  puts "  Dauer:[$Graph node get $Knoten Dauer] Woche(n)"
  set Ende [$Graph node get $Knoten Ende]
  puts "  Ende=[clock format $Ende -format $Zeitformat]"

  puts "vorherige(r) Knoten:"
  foreach Kante [$Graph arcs -in $Knoten] {
    puts "  [$Graph arc source $Kante]"
  }	

  puts "folgende(r) Knoten:"
  foreach Kante [$Graph arcs -out $Knoten] {
    puts "  [$Graph arc target $Kante]"
  }

  lappend AngezeigteKnoten $Knoten
  puts "-----------------"
}

proc TermineBerechnen {Modus Graph Knoten} {
  # Berechnet Beginn und Ende des Knoten
  global BerechneteKnoten
 
  if {$Knoten ni $BerechneteKnoten} {
    # Das maximale Ende aus allen vorherigen Knoten bestimmen
    set EndeMax 0
    foreach Kante [$Graph arcs -in $Knoten] {
      set KnotenVorher [$Graph arc source $Kante]
      set Ende [$Graph node get $KnotenVorher Ende]
      if {$Ende > $EndeMax} {
        set EndeMax $Ende
      }
    }
    if {$EndeMax != 0} {
      $Graph node set $Knoten Beginn $EndeMax
    }
    set Beginn [$Graph node get $Knoten Beginn]
    set Dauer [$Graph node get $Knoten Dauer]
    set Ende [clock add $Beginn $Dauer weeks]
    $Graph node set $Knoten Ende $Ende
    lappend BerechneteKnoten $Knoten
  }
}

proc Speichern {Graph Datei} {
  # Speichert den Graph in die Datei
  # Graph = Name des Graph
  # Datei = Dateiname
  set f [open $Datei w]
  puts $f [$Graph serialize]
  close $f
}

proc Oeffnen {Graph Datei} {
  # Oeffnet die Datei und speichert die Daten als Graph
  # Graph = Name des Graph
  # Datei = Dateiname
  if {[file exist $Datei] == 0} {return}
  set f [open $Datei r]
  gets $f Zeile
  $Graph deserialize $Zeile
  close $f
}

set Zeitformat "%d.%m.%Y"

::struct::graph Hausbau

# Knoten erzeugen
Hausbau node insert Start Bodenplatte Rohbau Dach Estrich Fenster Heizung Elektrik Bodenbelag Tapete Sanitaer Einzug Ende

# Eigenschaften der Knoten festlegen
Hausbau node set Start Beschreibung "Start"
Hausbau node set Start Dauer 0 ; # in Wochen
Hausbau node set Start Person ""
Hausbau node set Start Status ""
Hausbau node set Start Beginn [clock scan "00:00:00 2021-05-01"]
Hausbau node set Start Ende ""

Hausbau node set Bodenplatte Beschreibung "Bodenplatte giessen"
Hausbau node set Bodenplatte Dauer 2
Hausbau node set Bodenplatte Person "Anton"
Hausbau node set Bodenplatte Status "fertig"
Hausbau node set Bodenplatte Beginn ""
Hausbau node set Bodenplatte Ende ""

Hausbau node set Rohbau Beschreibung "Rohbau erstellen"
Hausbau node set Rohbau Dauer 8
Hausbau node set Rohbau Person "Berta"
Hausbau node set Rohbau Status "fertig"
Hausbau node set Rohbau Beginn ""
Hausbau node set Rohbau Ende ""

Hausbau node set Dach Beschreibung "Dach decken"
Hausbau node set Dach Dauer 1
Hausbau node set Dach Person "Caesar"
Hausbau node set Dach Status "fertig"
Hausbau node set Dach Beginn ""
Hausbau node set Dach Ende ""

Hausbau node set Estrich Beschreibung "Estrich auftragen"
Hausbau node set Estrich Dauer 8
Hausbau node set Estrich Person "Dora"
Hausbau node set Estrich Status "fertig"
Hausbau node set Estrich Beginn ""
Hausbau node set Estrich Ende ""

Hausbau node set Fenster Beschreibung "Fenster einbauen"
Hausbau node set Fenster Dauer 2
Hausbau node set Fenster Person "Emil"
Hausbau node set Fenster Status "fertig"
Hausbau node set Fenster Beginn ""
Hausbau node set Fenster Ende ""

Hausbau node set Heizung Beschreibung "Heizung einbauen"
Hausbau node set Heizung Dauer 3
Hausbau node set Heizung Person "Friedrich"
Hausbau node set Heizung Status "fertig"
Hausbau node set Heizung Beginn ""
Hausbau node set Heizung Ende ""

Hausbau node set Elektrik Beschreibung "Elektrik installieren"
Hausbau node set Elektrik Dauer 2
Hausbau node set Elektrik Person "Gustav"
Hausbau node set Elektrik Status "in Arbeit"
Hausbau node set Elektrik Beginn ""
Hausbau node set Elektrik Ende ""

Hausbau node set Bodenbelag Beschreibung "Bodenbelag verlegen"
Hausbau node set Bodenbelag Dauer 3
Hausbau node set Bodenbelag Person "Heinrich"
Hausbau node set Bodenbelag Status "in Arbeit"
Hausbau node set Bodenbelag Beginn ""
Hausbau node set Bodenbelag Ende ""

Hausbau node set Tapete Beschreibung "Tapete anbringen"
Hausbau node set Tapete Dauer 1
Hausbau node set Tapete Person "Ida"
Hausbau node set Tapete Status "noch nicht begonnen"
Hausbau node set Tapete Beginn ""
Hausbau node set Tapete Ende ""

Hausbau node set Sanitaer Beschreibung "Sanitaer montieren"
Hausbau node set Sanitaer Dauer 1
Hausbau node set Sanitaer Person "Karla"
Hausbau node set Sanitaer Status "noch nicht begonnen"
Hausbau node set Sanitaer Beginn ""
Hausbau node set Sanitaer Ende ""

Hausbau node set Einzug Beschreibung "Einzug"
Hausbau node set Einzug Dauer 1
Hausbau node set Einzug Person "Ludwig"
Hausbau node set Einzug Status "noch nicht begonnen"
Hausbau node set Einzug Beginn ""
Hausbau node set Einzug Ende ""

Hausbau node set Ende Beschreibung "Ende"
Hausbau node set Ende Dauer 0
Hausbau node set Ende Person ""
Hausbau node set Ende Status ""
Hausbau node set Ende Beginn ""
Hausbau node set Ende Ende ""

# Kanten erzeugen
Hausbau arc insert Start Bodenplatte
Hausbau arc insert Bodenplatte Rohbau
Hausbau arc insert Rohbau Dach
Hausbau arc insert Dach Fenster
Hausbau arc insert Dach Heizung
Hausbau arc insert Fenster Estrich
Hausbau arc insert Heizung Estrich
Hausbau arc insert Estrich Elektrik
Hausbau arc insert Estrich Bodenbelag
Hausbau arc insert Estrich Tapete
Hausbau arc insert Estrich Sanitaer
Hausbau arc insert Elektrik Einzug
Hausbau arc insert Bodenbelag Einzug
Hausbau arc insert Tapete Einzug
Hausbau arc insert Sanitaer Einzug
Hausbau arc insert Einzug Ende

# Termine berechnen
set BerechneteKnoten {}
Hausbau walk Start -type bfs -order pre -dir forward -command TermineBerechnen

# Graph anzeigen
set AngezeigteKnoten {}
Hausbau walk Start -type bfs -order pre -dir forward -command KnotenAnzeigen

puts "### speichern und oeffnen als Hausbau2 ###"
# Graph in Variable speichern und in einen neuen Graph importieren
Speichern Hausbau Graph.txt
::struct::graph Hausbau2
Oeffnen Hausbau2 Graph.txt
puts "Start:[clock format [Hausbau2 node get Start Beginn] -format $Zeitformat]"
puts "Ende:[clock format [Hausbau2 node get Ende Ende] -format $Zeitformat]"
